import math;
import hyperbolic_geometry;

size(500,500);
settings.outformat="pdf";

// Here we show the construction of a regular pentagon.
// This is done in analogy to the classical euclidean geometry.
// As we will see, this construction fails.

hyperbolic_point O = hyperbolic_point(0,0);

// The two points A, B given
// hyperbolic_point A = hyperbolic_point(3.0,54-30);
// hyperbolic_point B = hyperbolic_point(3.0,126-30);

hyperbolic_point A = hyperbolic_point(1.8,50);
hyperbolic_point B = hyperbolic_point(2.2,120);

// hyperbolic_point A = hyperbolic_point(0.8,30);
// hyperbolic_point B = hyperbolic_point(1.2,145);

// The line AB connection the A, B
hyperbolic_line AB = hyperbolic_line(A,B);

// Two circles with center {A,B} going through {B,A}
hyperbolic_circle C1 = hyperbolic_circle(A,B);
hyperbolic_circle C2 = hyperbolic_circle(B,A);

// Intersect the two circles, resulting in the points I1, I2
hyperbolic_point[] I = intersection(C1,C2);

// Put a line L through the two intersection points I1,I2
hyperbolic_line L = hyperbolic_line(I[0], I[1]);

// Intersect the line L with the segment S resulting in the midpoint M
hyperbolic_point M = intersection(AB, L);

// Normals
hyperbolic_line NA = hyperbolic_normal(AB,A);
hyperbolic_line NB = hyperbolic_normal(AB,B);

//
hyperbolic_circle D1 = hyperbolic_circle(A,M);
hyperbolic_circle D2 = hyperbolic_circle(B,M);

// Upper points
hyperbolic_point PNA = intersection(D1,NA)[0];
hyperbolic_point PNB = intersection(D2,NB)[1];

// Diagonal lines
hyperbolic_line GD1 = hyperbolic_line(PNA, B);
hyperbolic_line GD2 = hyperbolic_line(PNB, A);

// More circles
hyperbolic_circle E1 = hyperbolic_circle(PNA,A);
hyperbolic_circle E2 = hyperbolic_circle(PNB,B);

// Intersection points
hyperbolic_point X1 = intersection(E1, GD1)[0];
hyperbolic_point X2 = intersection(E2, GD2)[1];

// Circles
hyperbolic_circle F1 = hyperbolic_circle(A,X2);
hyperbolic_circle F2 = hyperbolic_circle(B,X1);

// And finally the golden ratio points
hyperbolic_point GR1 = intersection(F1,AB)[0];
hyperbolic_point GR2 = intersection(F2,AB)[1];

hyperbolic_circle CC1 = hyperbolic_circle(GR1, GR2);
hyperbolic_circle CC2 = hyperbolic_circle(GR1, A);
hyperbolic_circle CC3 = hyperbolic_circle(GR2, GR1);
hyperbolic_circle CC4 = hyperbolic_circle(GR2, B);

// The vertices of the regular pentagon
hyperbolic_point[] EP;
EP[0] = GR1;
EP[1] = GR2;
EP[2] = intersection(CC3,CC2)[0];
EP[3] = intersection(CC4,CC2)[0];
EP[4] = intersection(CC1,CC4)[1];

// The sides of the regular pentagon
hyperbolic_segment[] side;
side[0] = hyperbolic_segment(EP[0], EP[1]);
side[1] = hyperbolic_segment(EP[1], EP[2]);
side[2] = hyperbolic_segment(EP[2], EP[3]);
side[3] = hyperbolic_segment(EP[3], EP[4]);
side[4] = hyperbolic_segment(EP[4], EP[0]);

hyperbolic_polygon PG = hyperbolic_polygon(EP);

// And the inner one:
hyperbolic_segment[] side2;
for(int i=0; i<5; ++i) {
    side2[i] = hyperbolic_segment(EP[i%5], EP[(i+2)%5]);
}

hyperbolic_point[] EP2;
EP2[0] = intersection(side2[0], side2[1]);
EP2[1] = intersection(side2[1], side2[2]);
EP2[2] = intersection(side2[2], side2[3]);
EP2[3] = intersection(side2[3], side2[4]);
EP2[4] = intersection(side2[4], side2[0]);

hyperbolic_polygon PG2 = hyperbolic_polygon(EP2);

// And the third one
hyperbolic_segment[] side3;
for(int i=0; i<5; ++i) {
    side3[i] = hyperbolic_segment(EP2[i%5], EP2[(i+2)%5]);
}

// And draw all the stuff
draw(AB, black);
draw(C1, gray+opacity(0.2));
draw(C2, gray+opacity(0.2));
draw(L, gray+opacity(0.2));
draw(NA, gray);
draw(NB, gray);
draw(D1, gray+opacity(0.3));
draw(D2, gray+opacity(0.3));
draw(E1, gray+opacity(0.4));
draw(E2, gray+opacity(0.4));
draw(GD1, gray);
draw(GD2, gray);
draw(F1, gray+opacity(0.5));
draw(F2, gray+opacity(0.5));
draw(CC1, blue+opacity(0.5));
draw(CC2, blue+opacity(0.5));
draw(CC3, blue+opacity(0.5));
draw(CC4, blue+opacity(0.5));

dot(A, black);
dot(B, black);
dot(I, gray);
dot(M, gray);
dot(PNA, black);
dot(PNB, black);
dot(X1, gray);
dot(X2, gray);

draw(side3, black);
fill(PG2, orange+opacity(0.4));
draw(side2, black);
dot(PG2, red);
fill(PG, orange+opacity(0.4));
draw(PG, red);
dot(PG, red);

// Do some measurements whoch show that the regular pentagon is not that regular
// This is no surprise as the construction for the golden ratio fails.
for (int i = 0; i <= 4; ++i) {
    string s = string(side[i].length(), 4);
    label(s,(side[i].get_first_point().get_euclidean()+side[i].get_second_point().get_euclidean())/2);
}

for (int i = 0; i <= 4; ++i) {
    string s = string(angle(EP[(i)%5], EP[(i+1)%5], EP[(i+2)%5]), 4);
    label(s,EP[(i+1)%5].get_euclidean());
}

draw((1,1)--(1,-1)--(-1,-1)--(-1,1)--cycle);
draw(unitcircle);
